@@ -31,8 +31,8 @@ final class ColumnConsistencyTest extends TestCase
     protected function tearDown(): void
     {
         $csv = new SplFileObject(__DIR__.'/../test_files/foo.csv', 'w');
-        $csv->setCsvControl();
-        $csv->fputcsv(['john', 'doe', 'john.doe@example.com'], ',', '"');
+        $csv->setCsvControl(escape: '\\');
+        $csv->fputcsv(fields: ['john', 'doe', 'john.doe@example.com'], escape: '\\');
         unset($this->csv);
     }
 
