@@ -28,6 +28,7 @@ use TypeError;
 use function array_filter;
 use function array_reduce;
 use function get_defined_constants;
+use function is_bool;
 use function is_resource;
 use function is_string;
 use function json_encode;
@@ -120,17 +121,8 @@ final class JsonConverter
     private readonly string $indentation;
     /** @var Closure(array<int, T>): string */
     private readonly Closure $jsonEncodeChunk;
-
-    public static function create(): self
-    {
-        return new self(
-            flags: 0,
-            depth: 512,
-            indentSize: 4,
-            formatter: null,
-            chunkSize: 500
-        );
-    }
+    /** @var array<string> */
+    private array $indentationLevels = [];
 
     /**
      * @param int<1, max> $depth
@@ -140,8 +132,13 @@ final class JsonConverter
      *
      * @throws InvalidArgumentException
      */
-    private function __construct(int $flags, int $depth, int $indentSize, ?callable $formatter, int $chunkSize)
-    {
+    public function __construct(
+        int $flags = 0,
+        int $depth = 512,
+        int $indentSize = 4,
+        ?callable $formatter = null,
+        int $chunkSize = 500
+    ) {
         json_encode([], $flags & ~JSON_THROW_ON_ERROR, $depth);
 
         JSON_ERROR_NONE === ($errorCode = json_last_error()) || throw new InvalidArgumentException('The flags or the depth given are not valid JSON encoding parameters in PHP; '.json_last_error_msg(), $errorCode);
@@ -197,7 +194,7 @@ final class JsonConverter
         $inQuotes = false;
         $escape = false;
         $length = strlen($json);
-        $str = $this->indentation;
+        $str = [$this->indentation];
         for ($i = 0; $i < $length; $i++) {
             $char = $json[$i];
             if ('"' === $char && !$escape) {
@@ -205,16 +202,16 @@ final class JsonConverter
             }
 
             $escape = '\\' === $char && !$escape;
-            $str .= $inQuotes ? $char : match ($char) {
-                '{', '[' => $char."\n".str_repeat($this->indentation, ++$level),
-                '}', ']' =>  "\n".str_repeat($this->indentation, --$level).$char,
-                ',' => $char."\n".str_repeat($this->indentation, $level),
+            $str[] = $inQuotes ? $char : match ($char) {
+                '{', '[' => $char.($this->indentationLevels[++$level] ??= "\n".str_repeat($this->indentation, $level)),
+                '}', ']' =>  ($this->indentationLevels[--$level] ??= "\n".str_repeat($this->indentation, $level)).$char,
+                ',' => $char.($this->indentationLevels[$level] ??= "\n".str_repeat($this->indentation, $level)),
                 ':' => $char.' ',
                 default => $char,
             };
         }
 
-        return $str;
+        return implode('', $str);
     }
 
     /**
@@ -350,6 +347,26 @@ final class JsonConverter
         return new self($this->flags, $this->depth, $this->indentSize, $formatter, $this->chunkSize);
     }
 
+    /**
+     * Apply the callback if the given "condition" is (or resolves to) true.
+     *
+     * @param (callable($this): bool)|bool $condition
+     * @param callable($this): (self|null) $onSuccess
+     * @param ?callable($this): (self|null) $onFail
+     */
+    public function when(callable|bool $condition, callable $onSuccess, ?callable $onFail = null): self
+    {
+        if (!is_bool($condition)) {
+            $condition = $condition($this);
+        }
+
+        return match (true) {
+            $condition => $onSuccess($this),
+            null !== $onFail => $onFail($this),
+            default => $this,
+        } ?? $this;
+    }
+
     /**
      * Sends and makes the JSON structure downloadable via HTTP.
      *.
@@ -503,4 +520,23 @@ final class JsonConverter
             default => new self($this->flags, $this->depth, $indentSize, $this->formatter, $this->chunkSize),
         };
     }
+
+    /**
+     * DEPRECATION WARNING! This method will be removed in the next major point release.
+     *
+     * @see JsonConverter::__construct()
+     * @deprecated Since version 9.22.0
+     * @codeCoverageIgnore
+     */
+    #[Deprecated(message:'use League\Csv\JsonConverter::__construct() instead', since:'league/csv:9.22.0')]
+    public static function create(): self
+    {
+        return new self(
+            flags: 0,
+            depth: 512,
+            indentSize: 4,
+            formatter: null,
+            chunkSize: 500
+        );
+    }
 }
