@@ -18,6 +18,7 @@ use PHPUnit\Framework\Attributes\Group;
 use PHPUnit\Framework\TestCase;
 use SplFileObject;
 use TypeError;
+
 use function feof;
 use function fopen;
 use function fputcsv;
@@ -31,6 +32,7 @@ use function stream_context_get_options;
 use function stream_get_wrappers;
 use function stream_wrapper_register;
 use function stream_wrapper_unregister;
+
 use const STREAM_FILTER_READ;
 
 #[Group('csv')]
@@ -88,7 +90,7 @@ final class StreamTest extends TestCase
         ];
 
         foreach ($expected as $row) {
-            fputcsv($fp, $row);
+            fputcsv($fp, $row, escape: '');
         }
 
         $stream = Stream::createFromPath(
@@ -141,7 +143,7 @@ final class StreamTest extends TestCase
         $this->expectException(UnavailableFeature::class);
 
         $stream = Stream::createFromResource(STDOUT);
-        $stream->fputcsv(['foo', 'bar']);
+        $stream->fputcsv(['foo', 'bar'], escape: '');
         $stream->fseek(-1);
     }
 
@@ -203,6 +205,23 @@ final class StreamTest extends TestCase
         $stream = Stream::createFromPath('php://temp', 'r+');
         $stream->appendFilter($filtername, STREAM_FILTER_READ);
     }
+
+    public function testIterateOverLines(): void
+    {
+        $text = <<<TEXT
+Lorem ipsum dolor sit amet, consectetur adipiscing elit.
+Duis nec sapien felis, ac sodales nisl.
+Nulla vitae magna vitae purus aliquet consequat.
+TEXT;
+        $newText = '';
+        $file = Stream::createFromString($text);
+        $file->setMaxLineLen(20);
+        foreach ($file as $line) {
+            $newText .= $line."\n";
+        }
+        self::assertStringContainsString('Lorem ipsum dolor s', $newText);
+        self::assertSame(20, $file->getMaxLineLen());
+    }
 }
 
 final class StreamWrapper
@@ -226,7 +245,7 @@ final class StreamWrapper
         }
     }
 
-    public function stream_open(string $path, string $mode, int $options, string &$opened_path = null): bool
+    public function stream_open(string $path, string $mode, int $options, ?string &$opened_path = null): bool
     {
         $options = stream_context_get_options($this->context);
         if (!isset($options[self::PROTOCOL]['stream'])) {
@@ -239,7 +258,7 @@ final class StreamWrapper
     }
 
     /**
-     * @param int<0, max> $count
+     * @param int<1, max> $count
      */
     public function stream_read(int $count): string|false
     {
