@@ -13,6 +13,8 @@ declare(strict_types=1);
 
 namespace League\Csv;
 
+use Stringable;
+
 /**
  * A class to insert records into a CSV Document.
  */
@@ -23,8 +25,10 @@ interface TabularDataWriter
      *
      * @see TabularDataWriter::insertOne
      *
+     * @param iterable<array<null|int|float|string|Stringable>> $records
+     *
      * @throws CannotInsertRecord If the record can not be inserted
-     * @throws Exception          If the record can not be inserted
+     * @throws Exception If the record can not be inserted
      */
     public function insertAll(iterable $records): int;
 
@@ -34,8 +38,10 @@ interface TabularDataWriter
      * A record is an array that can contain scalar type values, NULL values
      * or objects implementing the __toString method.
      *
+     * @param array<null|int|float|string|Stringable> $record
+     *
      * @throws CannotInsertRecord If the record can not be inserted
-     * @throws Exception          If the record can not be inserted
+     * @throws Exception If the record can not be inserted
      */
     public function insertOne(array $record): int;
 }
